@@ -2,7 +2,9 @@
  * See header file for descriptions.
  *
  * liblognorm - a fast samples-based log normalization library
- * Copyright 2010 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2013 by Rainer Gerhards and Adiscon GmbH.
+ *
+ * Modified by Pavel Levshin (pavel@levshin.spb.ru) in 2013
  *
  * This file is part of liblognorm.
  *
@@ -23,6 +25,7 @@
  * A copy of the LGPL v2.1 can be found in the file "COPYING" in this distribution.
  */
 #include "config.h"
+#include <string.h>
 
 #include "liblognorm.h"
 #include "lognorm.h"
@@ -87,6 +90,8 @@ ln_exitCtx(ln_ctx ctx)
 		ln_deletePTree(ctx->ptree);
 	if(ctx->rulePrefix != NULL)
 		es_deleteStr(ctx->rulePrefix);
+	if(ctx->pas != NULL)
+		ln_deleteAnnotSet(ctx->pas);
 	free(ctx);
 done:
 	return r;
@@ -94,7 +99,7 @@ done:
 
 
 int
-ln_setDebugCB(ln_ctx ctx, void (*cb)(void*, char*, size_t), void *cookie)
+ln_setDebugCB(ln_ctx ctx, void (*cb)(void*, const char*, size_t), void *cookie)
 {
 	int r = 0;
 
@@ -107,7 +112,7 @@ done:
 
 
 int
-ln_loadSample(ln_ctx ctx, char *buf)
+ln_loadSample(ln_ctx ctx, const char *buf)
 {
     // Something bad happened - no new sample
     if (ln_processSamp(ctx, buf, strlen(buf)) == NULL) {
@@ -118,7 +123,7 @@ ln_loadSample(ln_ctx ctx, char *buf)
 
 
 int
-ln_loadSamples(ln_ctx ctx, char *file)
+ln_loadSamples(ln_ctx ctx, const char *file)
 {
 	int r = 0;
 	struct ln_sampRepos *repo;
@@ -139,9 +144,3 @@ done:
 	return r;
 }
 
-
-void
-ln_setEECtx(ln_ctx ctx, ee_ctx eectx)
-{
-	ctx->eectx = eectx;
-}
