@@ -36,7 +36,9 @@
  *//*
  *
  * liblognorm - a fast samples-based log normalization library
- * Copyright 2010 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2010-2013 by Rainer Gerhards and Adiscon GmbH.
+ *
+ * Modified by Pavel Levshin (pavel@levshin.spb.ru) in 2013
  *
  * This file is part of liblognorm.
  *
@@ -57,22 +59,19 @@
  * A copy of the LGPL v2.1 can be found in the file "COPYING" in this distribution.
  */
 #ifndef LIBLOGNORM_H_INCLUDED
-#define	LIBLOGNORM_H_INCLUDED
+#define LIBLOGNORM_H_INCLUDED
 #include <stdlib.h>	/* we need size_t */
-#include <libee/libee.h>
+#include <json.h>
+#include <libestr.h>
 
 /* error codes */
 #define LN_NOMEM -1
 #define LN_INVLDFDESCR -1
-
-/* event_t needs to come from libcee, or whatever it will be called. We
- * provide a dummy to be able to compile the initial skeletons.
- */
-typedef void * event_t;
+#define LN_WRONGPARSER -1000
 
 /**
  * The library context descriptor.
- * This is used to permit multiple independednt instances of the
+ * This is used to permit multiple independent instances of the
  * library to be called within a single program. This is most
  * useful for plugin-based architectures.
  */
@@ -114,15 +113,6 @@ ln_ctx ln_initCtx(void);
 int ln_exitCtx(ln_ctx ctx);
 
 
-/**
- * Set the libee context to be used by this liblognorm context.
- *
- * @param ctx context to be modified
- * @param eectx	libee context
- */
-void ln_setEECtx(ln_ctx ctx, ee_ctx eectx);
-
-
 /**
  * Set a debug message handler (callback).
  *
@@ -155,7 +145,7 @@ void ln_setEECtx(ln_ctx ctx, ee_ctx eectx);
  *
  * @return Returns zero on success, something else otherwise.
  */
-int ln_setDebugCB(ln_ctx ctx, void (*cb)(void*, char*, size_t), void *cookie);
+int ln_setDebugCB(ln_ctx ctx, void (*cb)(void*, const char*, size_t), void *cookie);
 
 
 /**
@@ -179,7 +169,7 @@ void ln_enableDebug(ln_ctx ctx, int i);
  * @return Returns zero on success, something else otherwise.
  */
 int
-ln_loadSample(ln_ctx ctx, char *buf);
+ln_loadSample(ln_ctx ctx, const char *buf);
 
 /**
  * Load a (log) sample file.
@@ -194,7 +184,7 @@ ln_loadSample(ln_ctx ctx, char *buf);
  *
  * @return Returns zero on success, something else otherwise.
  */
-int ln_loadSamples(ln_ctx ctx, char *file);
+int ln_loadSamples(ln_ctx ctx, const char *file);
 
 /**
  * Normalize a message.
@@ -215,14 +205,13 @@ int ln_loadSamples(ln_ctx ctx, char *file);
  * must be provided.
  *
  * @param[in] ctx The library context to use.
- * @param[in] msg The message string (see note above).
- * @param[in] lenmsg The length of the message in bytes.
- * @param[out] event A new event record or NULL if an error occured. <b>Must be
+ * @param[in] str The message string (see note above).
+ * @param[in] strLen The length of the message in bytes.
+ * @param[out] json_p A new event record or NULL if an error occured. <b>Must be
  *                   destructed if no longer needed.</b>
  *
  * @return Returns zero on success, something else otherwise.
  */
-int ln_normalizeMsg(ln_ctx ctx, char *msg, size_t lenmsg, event_t *event);
-int ln_normalize(ln_ctx ctx, es_str_t *str, struct ee_event **event);
+int ln_normalize(ln_ctx ctx, const char *str, size_t strLen, struct json_object **json_p);
 
 #endif /* #ifndef LOGNORM_H_INCLUDED */
