@@ -12,7 +12,7 @@
  *
  *//*
  * liblognorm - a fast samples-based log normalization library
- * Copyright 2010-2011 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2010-2013 by Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of liblognorm.
  *
@@ -36,17 +36,17 @@
 #include <string.h>
 #include <getopt.h>
 #include <libestr.h>
-#include <libee/libee.h>
+#include <json.h>
 #include "liblognorm.h"
 #include "ptree.h"
 #include "lognorm.h"
+#include "enc.h"
 
 static ln_ctx ctx;
-static ee_ctx eectx;
 
 static int verbose = 0;
 static int parsedOnly = 0;	/**< output unparsed messages? */
-static int flatTags = 0;	/**< output unparsed messages? */
+static int flatTags = 0;	/**< print event.tags in JSON? */
 static FILE *fpDOT;
 static es_str_t *encFmt = NULL; /**< a format string for encoder use */
 static es_str_t *mandatoryTag = NULL; /**< tag which must be given so that mesg will
@@ -54,7 +54,7 @@ static es_str_t *mandatoryTag = NULL; /**< tag which must be given so that mesg
 static enum { f_syslog, f_json, f_xml, f_csv } outfmt = f_syslog;
 
 void
-dbgCallBack(void __attribute__((unused)) *cookie, char *msg,
+dbgCallBack(void __attribute__((unused)) *cookie, const char *msg,
 	    size_t __attribute__((unused)) lenMsg)
 {
 	printf("liblognorm: %s\n", msg);
@@ -71,32 +71,61 @@ void errout(char *errmsg)
  * of the string on every call.
  */
 static inline void
-outputEvent(struct ee_event *event)
+outputEvent(struct json_object *json)
 {
 	char *cstr;
 	es_str_t *str = NULL;
 
 	switch(outfmt) {
 	case f_json:
-		ee_fmtEventToJSON(event, &str);
+		if(!flatTags) {
+			json_object_object_del(json, "event.tags");
+		}
+		cstr = (char*)json_object_to_json_string(json);
 		break;
 	case f_syslog:
-		ee_fmtEventToRFC5424(event, &str);
+		ln_fmtEventToRFC5424(json, &str);
 		break;
 	case f_xml:
-		ee_fmtEventToXML(event, &str);
+		ln_fmtEventToXML(json, &str);
 		break;
 	case f_csv:
-		ee_fmtEventToCSV(event, &str, encFmt);
+	ln_fmtEventToCSV(json, &str, encFmt);
 		break;
 	}
-	cstr = es_str2cstr(str, NULL);
+	if (str != NULL)
+		cstr = es_str2cstr(str, NULL);
 	if(verbose > 0) printf("normalized: '%s'\n", cstr);
 	printf("%s\n", cstr);
-	free(cstr);
+	if (str != NULL)
+		free(cstr);
 	es_deleteStr(str);
 }
 
+/* test if the tag exists */
+static int
+eventHasTag(struct json_object *json, const char *tag)
+{
+	struct json_object *tagbucket, *tagObj;
+	int i;
+	const char *tagCstr;
+	
+	if (tag == NULL)
+		return 1;
+	if ((tagbucket = json_object_object_get(json, "event.tags")) != NULL) {
+		if (json_object_get_type(tagbucket) == json_type_array) {
+			for (i = json_object_array_length(tagbucket) - 1; i >= 0; i--) {
+				tagObj = json_object_array_get_idx(tagbucket, i);
+				tagCstr = json_object_get_string(tagObj);
+				if (!strcmp(tag, tagCstr))
+					return 1;
+			}
+		}
+	}
+	if (verbose > 1)
+		printf("Mandatory tag '%s' has not been found\n", tag);
+	return 0;
+}
 
 /* normalize input data
  */
@@ -105,44 +134,41 @@ normalize(void)
 {
 	FILE *fp = stdin;
 	char buf[10*1024];
-	es_str_t *str;
-	struct ee_event *event = NULL;
-	es_str_t *constUnparsed;
+	struct json_object *json = NULL;
 	long long unsigned numUnparsed = 0;
 	long long unsigned numWrongTag = 0;
-
-	constUnparsed = es_newStrFromBuf("unparsed-data", sizeof("unparsed-data") - 1);
+	char *mandatoryTagCstr = NULL;
+	
+	if (mandatoryTag != NULL) {
+		mandatoryTagCstr = es_str2cstr(mandatoryTag, NULL);
+	}
 
 	while((fgets(buf, sizeof(buf), fp)) != NULL) {
 		buf[strlen(buf)-1] = '\0';
 		if(strlen(buf) > 0 && buf[strlen(buf)-1] == '\r')
 			buf[strlen(buf)-1] = '\0';
 		if(verbose > 0) printf("To normalize: '%s'\n", buf);
-		str = es_newStrFromCStr(buf, strlen(buf));
-		ln_normalize(ctx, str, &event);
-		//printf("normalize result: %d\n", ln_normalizeRec(ctx, ctx->ptree, str, 0, &event));
-		if(event != NULL) {
-			if(   mandatoryTag == NULL
-			   || (mandatoryTag != NULL && ee_EventHasTag(event, mandatoryTag))) {
-				if(   parsedOnly == 1
-				   && ee_getEventField(event, constUnparsed) != NULL){
+		ln_normalize(ctx, buf, strlen(buf), &json);
+		if(json != NULL) {
+			if(eventHasTag(json, mandatoryTagCstr)) {
+				if( parsedOnly == 1
+						&& json_object_object_get(json, "unparsed-data") != NULL) {
 					numUnparsed++;
 				} else {
-					outputEvent(event);
+					outputEvent(json);
 				}
 			} else {
 				numWrongTag++;
 			}
-			ee_deleteEvent(event);
-			event = NULL;
+			json_object_put(json);
+			json = NULL;
 		}
-		es_deleteStr(str);
 	}
 	if(numUnparsed > 0)
 		fprintf(stderr, "%llu unparsable entries\n", numUnparsed);
 	if(numWrongTag > 0)
 		fprintf(stderr, "%llu entries with wrong tag dropped\n", numWrongTag);
-	es_deleteStr(constUnparsed);
+	free(mandatoryTagCstr);
 }
 
 
@@ -214,18 +240,10 @@ int main(int argc, char *argv[])
 		errout("Could not initialize liblognorm context");
 	}
 
-	if((eectx = ee_initCtx()) == NULL) {
-		errout("Could not initialize libee context");
-	}
-	if(flatTags) {
-		ee_setFlags(eectx, EE_CTX_FLAG_INCLUDE_FLAT_TAGS);
-	}
-
 	if(verbose) {
 		ln_setDebugCB(ctx, dbgCallBack, NULL);
 		ln_enableDebug(ctx, 1);
 	}
-	ln_setEECtx(ctx, eectx);
 
 	ln_loadSamples(ctx, repository);
 
@@ -242,5 +260,7 @@ int main(int argc, char *argv[])
 	normalize();
 
 	ln_exitCtx(ctx);
+	if (encFmt != NULL)
+		free(encFmt);
 	return 0;
 }
