@@ -3,7 +3,9 @@
  * @brief The parse tree object.
  * @class ln_ptree ptree.h
  *//*
- * Copyright 2010 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2013 by Rainer Gerhards and Adiscon GmbH.
+ *
+ * Modified by Pavel Levshin (pavel@levshin.spb.ru) in 2013
  *
  * This file is meant to be included by applications using liblognorm.
  * For lognorm library files themselves, include "lognorm.h".
@@ -29,7 +31,6 @@
 #ifndef LIBLOGNORM_PTREE_H_INCLUDED
 #define	LIBLOGNORM_PTREE_H_INCLUDED
 #include <libestr.h>
-#include <libee/libee.h>
 
 typedef struct ln_ptree ln_ptree; /**< the parse tree object */
 typedef struct ln_fieldList_s ln_fieldList_t;
@@ -44,12 +45,13 @@ typedef struct ln_fieldList_s ln_fieldList_t;
  * optimize it so that frequently used fields are moved "up" towards
  * the root of the list. In any case, we do NOT expect this list to
  * be long, as the parser should already have gotten quite specific when
- * we hit a field.
+ * we hit a fieldconst .
  */
 struct ln_fieldList_s {
 	es_str_t *name;		/**< field name */
 	es_str_t *data;		/**< extra data to be passed to parser */
-	int (*parser)(ee_ctx, es_str_t*, es_size_t*, es_str_t*, struct ee_value**);
+	int (*parser)(const char*, size_t, size_t*, es_str_t*, size_t*,
+			struct json_object **);
 				/**< parser to use */
 	ln_ptree *subtree;	/**< subtree to follow if parser succeeded */
 	ln_fieldList_t *next;	/**< list housekeeping, next node (or NULL) */
@@ -68,7 +70,7 @@ struct ln_ptree {
 	struct {
 		unsigned isTerminal:1;	/**< designates this node a terminal sequence? */
 	} flags;
-	struct ee_tagbucket *tags;	/* tags to assign to events of this type */
+	struct json_object *tags;	/* tags to assign to events of this type */
 	/* the respresentation below requires a lof of memory but is
 	 * very fast. As an alternate approach, we can use a hash table
 	 * where we ignore control characters. That should work quite well.
@@ -122,30 +124,6 @@ void ln_deletePTree(struct ln_ptree *tree);
 int ln_addFDescrToPTree(struct ln_ptree **tree, ln_fieldList_t *node);
 
 
-/**
- * Traverse a (sub) tree according to a string.
- *
- * This functions traverses the provided tree according to the
- * provided string. It navigates to the deepest node possible.
- * Then, it returns this node as well as the position until which
- * the string could be parsed. If there is no match at all,
- * NULL is returned instead of a tree node. Note that this is
- * different from the case where the root of the subtree is
- * returned. In that case, there was at least a single match
- * inside that root.
- * @memberof ln_ptree
- *
- * @param[in] subtree root of subtree to traverse
- * @param[in] str string to parse
- * @param[in/out] parsedTo on entry: start position within string,
- * 	          on exist position of first unmatched byte
- *
- * @return pointer to found tree node or NULL if there was no match at all
- */
-struct ln_ptree* ln_traversePTree(struct ln_ptree *subtree,
-                               es_str_t *str, es_size_t *parsedTo);
-
-
 /**
  * Add a literal to a ptree.
  * Creates new tree nodes as necessary.
@@ -158,7 +136,7 @@ struct ln_ptree* ln_traversePTree(struct ln_ptree *subtree,
  * @return NULL on error, otherwise pointer to deepest tree added
  */
 struct ln_ptree*
-ln_addPTree(struct ln_ptree *tree, es_str_t *str, es_size_t offs);
+ln_addPTree(struct ln_ptree *tree, es_str_t *str, size_t offs);
 
 
 /**
@@ -202,6 +180,6 @@ void ln_genDotPTreeGraph(struct ln_ptree *tree, es_str_t **str);
  * @return NULL on error, otherwise the ptree leaf that
  *         corresponds to the parameters passed.
  */
-struct ln_ptree * ln_buildPTree(struct ln_ptree *tree, es_str_t *str, es_size_t offs);
+struct ln_ptree * ln_buildPTree(struct ln_ptree *tree, es_str_t *str, size_t offs);
 
 #endif /* #ifndef LOGNORM_PTREE_H_INCLUDED */
