@@ -2,6 +2,8 @@
  *
  * Copyright 2010 by Rainer Gerhards and Adiscon GmbH.
  *
+ * Modified by Pavel Levshin (pavel@levshin.spb.ru) in 2013
+ *
  * This file is part of liblognorm.
  *
  * This library is free software; you can redistribute it and/or
@@ -27,16 +29,15 @@
 #include <string.h>
 #include <assert.h>
 #include <ctype.h>
-#include <libee/libee.h>
-#include <libee/primitivetype.h>
 
 #include "liblognorm.h"
 #include "lognorm.h"
 #include "samp.h"
 #include "internal.h"
+#include "parser.h"
 
 struct ln_sampRepos*
-ln_sampOpen(ln_ctx __attribute((unused)) ctx, char *name)
+ln_sampOpen(ln_ctx __attribute((unused)) ctx, const char *name)
 {
 	struct ln_sampRepos *repo = NULL;
 	FILE *fp;
@@ -162,29 +163,34 @@ parseFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 
 	node->isIPTables = 0; /* first assume no special parser is used */
 	if(!es_strconstcmp(*str, "date-rfc3164")) {
-		node->parser = ee_parseRFC3164Date;
+		node->parser = ln_parseRFC3164Date;
 	} else if(!es_strconstcmp(*str, "date-rfc5424")) {
-		node->parser = ee_parseRFC5424Date;
+		node->parser = ln_parseRFC5424Date;
 	} else if(!es_strconstcmp(*str, "number")) {
-		node->parser = ee_parseNumber;
+		node->parser = ln_parseNumber;
 	} else if(!es_strconstcmp(*str, "ipv4")) {
-		node->parser = ee_parseIPv4;
+		node->parser = ln_parseIPv4;
 	} else if(!es_strconstcmp(*str, "word")) {
-		node->parser = ee_parseWord;
+		node->parser = ln_parseWord;
+	} else if(!es_strconstcmp(*str, "rest")) {
+		node->parser = ln_parseRest;
 	} else if(!es_strconstcmp(*str, "quoted-string")) {
-		node->parser = ee_parseQuotedString;
+		node->parser = ln_parseQuotedString;
 	} else if(!es_strconstcmp(*str, "date-iso")) {
-		node->parser = ee_parseISODate;
+		node->parser = ln_parseISODate;
 	} else if(!es_strconstcmp(*str, "time-24hr")) {
-		node->parser = ee_parseTime24hr;
+		node->parser = ln_parseTime24hr;
 	} else if(!es_strconstcmp(*str, "time-12hr")) {
-		node->parser = ee_parseTime12hr;
+		node->parser = ln_parseTime12hr;
 	} else if(!es_strconstcmp(*str, "iptables")) {
 		node->parser = NULL;
 		node->isIPTables = 1;
 	} else if(!es_strconstcmp(*str, "char-to")) {
 		// TODO: check extra data!!!! (very important)
-		node->parser = ee_parseCharTo;
+		node->parser = ln_parseCharTo;
+	} else if(!es_strconstcmp(*str, "char-sep")) {
+		// TODO: check extra data!!!! (very important)
+		node->parser = ln_parseCharSeparated;
 	} else {
 		cstr = es_str2cstr(*str, NULL);
 		ln_dbgprintf(ctx, "ERROR: invalid field type '%s'", cstr);
@@ -201,9 +207,7 @@ parseFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 		while(i < lenBuf) {
 			if(buf[i] == '%') {
 				++i;
-				if(i == lenBuf || buf[i] != '%') {
-					break; /* end of field */
-				}
+				break; /* end of field */
 			}
 			CHKR(es_addChar(&node->data, buf[i++]));
 		}
@@ -260,8 +264,6 @@ parseLiteral(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 		CHKR(es_addChar(str, buf[i]));
 		++i;
 	}
-	if(es_strlen(*str) == 0)
-		goto done;
 
 	es_unescapeStr(*str);
 	if(ctx->debug) {
@@ -293,7 +295,7 @@ done:	return r;
  * @returns the new subtree root (or NULL in case of error)
  */
 static inline int
-addSampToTree(ln_ctx ctx, es_str_t *rule, struct ee_tagbucket *tagBucket)
+addSampToTree(ln_ctx ctx, es_str_t *rule, struct json_object *tagBucket)
 {
 	int r;
 	struct ln_ptree* subtree;
@@ -304,14 +306,19 @@ addSampToTree(ln_ctx ctx, es_str_t *rule, struct ee_tagbucket *tagBucket)
 	CHKN(str = es_newStr(256));
 	i = 0;
 	while(i < es_strlen(rule)) {
-ln_dbgprintf(ctx, "addSampToTree %d of %d", i, es_strlen(rule));
+		ln_dbgprintf(ctx, "addSampToTree %d of %d", i, es_strlen(rule));
 		CHKR(parseLiteral(ctx, &subtree, rule, &i, &str));
-		if(es_strlen(str) == 0) {
-			/* we had no literal, so let's parse a field description */
+		/* After the literal there can be field only*/
+		if (i < es_strlen(rule)) {
 			CHKR(parseFieldDescr(ctx, &subtree, rule, &i, &str));
+			if (i == es_strlen(rule)) {
+				/* finish the tree with empty literal to avoid false merging*/
+				CHKR(parseLiteral(ctx, &subtree, rule, &i, &str));
+			}
 		}
 	}
-ln_dbgprintf(ctx, "end addSampToTree %d of %d", i, es_strlen(rule));
+
+	ln_dbgprintf(ctx, "end addSampToTree %d of %d", i, es_strlen(rule));
 	/* we are at the end of rule processing, so this node is a terminal */
 	subtree->flags.isTerminal = 1;
 	subtree->tags = tagBucket;
@@ -334,7 +341,7 @@ done:
  * @returns 0 on success, something else otherwise
  */
 static inline int
-getLineType(char *buf, es_size_t lenBuf, es_size_t *offs, es_str_t **str)
+getLineType(const char *buf, es_size_t lenBuf, es_size_t *offs, es_str_t **str)
 {
 	int r;
 	es_size_t i;
@@ -364,7 +371,7 @@ done:	return r;
  * @returns 0 on success, something else otherwise
  */
 static inline int
-getPrefix(char *buf, es_size_t lenBuf, es_size_t offs, es_str_t **str)
+getPrefix(const char *buf, es_size_t lenBuf, es_size_t offs, es_str_t **str)
 {
 	int r;
 
@@ -374,7 +381,7 @@ getPrefix(char *buf, es_size_t lenBuf, es_size_t offs, es_str_t **str)
 		es_emptyStr(*str);
 	}
 
-	r = es_addBuf(str, buf + offs, lenBuf - offs);
+	r = es_addBuf(str, (char*)buf + offs, lenBuf - offs);
 done:	return r;
 }
 
@@ -390,9 +397,9 @@ done:	return r;
  * @returns 0 on success, something else otherwise
  */
 static inline int
-extendPrefix(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t offs)
+extendPrefix(ln_ctx ctx, const char *buf, es_size_t lenBuf, es_size_t offs)
 {
-	return es_addBuf(&ctx->rulePrefix, buf+offs, lenBuf - offs);
+	return es_addBuf(&ctx->rulePrefix, (char*)buf+offs, lenBuf - offs);
 }
 
 
@@ -405,18 +412,20 @@ extendPrefix(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t offs)
  * @returns 0 on success, something else otherwise
  */
 static inline int
-addTagStrToBucket(ln_ctx ctx, es_str_t *tagname, struct ee_tagbucket **tagBucket)
+addTagStrToBucket(ln_ctx ctx, es_str_t *tagname, struct json_object **tagBucket)
 {
 	int r = -1;
 	char *cstr;
+	struct json_object *tag; 
 
 	if(*tagBucket == NULL) {
-		CHKN(*tagBucket = ee_newTagbucket(ctx->eectx));
+		CHKN(*tagBucket = json_object_new_array());
 	}
 	cstr = es_str2cstr(tagname, NULL);
 	ln_dbgprintf(ctx, "tag found: '%s'", cstr);
+	CHKN(tag = json_object_new_string(cstr));
+	json_object_array_add(*tagBucket, tag);
 	free(cstr);
-	CHKR(ee_addTagToBucket(*tagBucket, tagname));
 	r = 0;
 
 done:	return r;
@@ -436,7 +445,7 @@ done:	return r;
  * @returns 0 on success, something else otherwise
  */
 static inline int
-processTags(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t *poffs, struct ee_tagbucket **tagBucket)
+processTags(ln_ctx ctx, const char *buf, es_size_t lenBuf, es_size_t *poffs, struct json_object **tagBucket)
 {
 	int r = -1;
 	es_str_t *str = NULL;
@@ -448,6 +457,7 @@ processTags(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t *poffs, struct ee
 		if(buf[i] == ',') {
 			/* end of this tag */
 			CHKR(addTagStrToBucket(ctx, str, tagBucket));
+			es_deleteStr(str);
 			str = NULL;
 		} else {
 			if(str == NULL) {
@@ -464,6 +474,7 @@ processTags(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t *poffs, struct ee
 
 	if(str != NULL) {
 		CHKR(addTagStrToBucket(ctx, str, tagBucket));
+		es_deleteStr(str);
 	}
 
 	*poffs = i;
@@ -483,11 +494,11 @@ done:	return r;
  * @returns 0 on success, something else otherwise
  */
 static inline int
-processRule(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t offs)
+processRule(ln_ctx ctx, const char *buf, es_size_t lenBuf, es_size_t offs)
 {
 	int r = -1;
 	es_str_t *str;
-	struct ee_tagbucket *tagBucket = NULL;
+	struct json_object *tagBucket = NULL;
 
 	ln_dbgprintf(ctx, "sample line to add: '%s'\n", buf+offs);
 	CHKR(processTags(ctx, buf, lenBuf, &offs, &tagBucket));
@@ -502,7 +513,7 @@ processRule(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t offs)
 	} else {
 		CHKN(str = es_strdup(ctx->rulePrefix));
 	}
-	CHKR(es_addBuf(&str, buf + offs, lenBuf - offs));
+	CHKR(es_addBuf(&str, (char*)buf + offs, lenBuf - offs));
 	addSampToTree(ctx, str, tagBucket);
 	es_deleteStr(str);
 	r = 0;
@@ -522,7 +533,7 @@ done:	return r;
  * @returns 0 on success, something else otherwise
  */
 static inline int
-getFieldName(ln_ctx __attribute__((unused)) ctx, char *buf, es_size_t lenBuf, es_size_t *offs, es_str_t **strTag)
+getFieldName(ln_ctx __attribute__((unused)) ctx, const char *buf, es_size_t lenBuf, es_size_t *offs, es_str_t **strTag)
 {
 	int r = -1;
 	es_size_t i;
@@ -552,7 +563,7 @@ done:	return r;
  * @param[in/out] offs on entry: offset first unprocessed position
  */
 static inline void
-skipWhitespace(ln_ctx __attribute__((unused)) ctx, char *buf, es_size_t lenBuf, es_size_t *offs)
+skipWhitespace(ln_ctx __attribute__((unused)) ctx, const char *buf, es_size_t lenBuf, es_size_t *offs)
 {
 	while(*offs < lenBuf && isspace(buf[*offs])) {
 		(*offs)++;
@@ -580,7 +591,7 @@ skipWhitespace(ln_ctx __attribute__((unused)) ctx, char *buf, es_size_t lenBuf,
  * @returns 0 on success, something else otherwise
  */
 static inline int
-getAnnotationOp(ln_ctx ctx, ln_annot *annot, char *buf, es_size_t lenBuf, es_size_t *offs)
+getAnnotationOp(ln_ctx ctx, ln_annot *annot, const char *buf, es_size_t lenBuf, es_size_t *offs)
 {
 	int r = -1;
 	es_size_t i;
@@ -642,7 +653,7 @@ fail:	return -1;
  * @returns 0 on success, something else otherwise
  */
 static inline int
-processAnnotate(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t offs)
+processAnnotate(ln_ctx ctx, const char *buf, es_size_t lenBuf, es_size_t offs)
 {
 	int r;
 	es_str_t *tag = NULL;
@@ -651,7 +662,7 @@ processAnnotate(ln_ctx ctx, char *buf, es_size_t lenBuf, es_size_t offs)
 	ln_dbgprintf(ctx, "sample annotation to add: '%s'", buf+offs);
 	CHKR(getFieldName(ctx, buf, lenBuf, &offs, &tag));
 	skipWhitespace(ctx, buf, lenBuf, &offs);
-	if(buf[offs] != ':') {
+	if(buf[offs] != ':' || tag == NULL) {
 		ln_dbgprintf(ctx, "invalid tag field in annotation, line is '%s'", buf);
 		r=-1;
 		goto done;
@@ -671,7 +682,7 @@ done:	return r;
 }
 
 struct ln_samp *
-ln_processSamp(ln_ctx ctx, char *buf, es_size_t lenBuf)
+ln_processSamp(ln_ctx ctx, const char *buf, es_size_t lenBuf)
 {
 	struct ln_samp *samp = NULL;
     es_str_t *typeStr = NULL;
